/**
 * NewsFeed handler module
 */

'use strict';

(function() {

/** Default config */
var defaults = {
  preloadThumbnails: false
};

var config;

TS.Modules.NewsFeed = function() {
  TS.Modules.Feed.call(this);

  config = $.extend({}, defaults, TS.Config.NewsFeed || {});

  if (TS.currentFeedSource === 'dummy') {
    this.url = 'json/newsChannels.json.php?lang={lang}';
  } else if (TS.currentFeedSource === 'development') {
    this.url = 'http://rn-1056-tvapps.qa.goal.com/feed/section/list?edition={lang}&format=tvapps';
  } else if (TS.currentFeedSource === 'preproduction') {
    this.url = 'http://cdnpreview.goal.com/feed/section/list?edition={lang}&format=tvapps';
  } else {
    this.url = 'http://www.goal.com/feed/section/list?edition={lang}&format=tvapps';
  }

  if (TS.Platform.corsSupport) {
    this.format = 'json';
  } else {
    this.format = 'jsonp';
    this.url += '&callback={callback}';
  }

  this.channels = [];
  this.isReady = false;

  /**
   * pull available channels.
   * @fires ready on success
   * @fires error on failure
   * @param lang {string}language code
   */
  this.pullChannels = function(lang) {
    this.isReady = false;
    lang = TS.Config.edition();

    this.channels = [];
    this.pull({'lang': lang});
    this.lang = lang;
    return this;
  };

  this.processData = function(data) {
    this.channels = [];
    
    data.forEach(function(e, i, a) {
      this.channels.push(new NewsChannel(e.id, e.title, this));
    }.bind(this));

    this.isReady = true;
    this.latestNews = this.channels[0];
    $.trace.info('NewsFeed: ready');
    this.isReady = true;
    return this.channels;
  }.bind(this);
};

var NewsChannel = function(id, name, newsFeed) {
  TS.Modules.Feed.call(this);

  if (TS.currentFeedSource === 'dummy') {
    this.url = 'json/news.json.php?lang={lang}&channelID={channelID}';
  } else if (TS.currentFeedSource === 'development') {
    this.url = 'http://rn-1056-tvapps.qa.goal.com/feed/news/list?edition={lang}&section={channelID}&format=tvapps';
  } else if (TS.currentFeedSource === 'preproduction') {
    this.url = 'http://cdnpreview.goal.com/feed/news/list?edition={lang}&section={channelID}&format=tvapps';
  } else {
    this.url = 'http://www.goal.com/feed/news/list?edition={lang}&section={channelID}&format=tvapps';
  }

  if (TS.Platform.corsSupport) {
    this.format = 'json';
  } else {
    this.format = 'jsonp';
    this.url += '&callback={callback}';
  }

  this.id = id;
  this.name = name;
  this.news = [];
  this.parentObject = this.newsFeed = newsFeed;
  this.isReady = false;
  this.isLoaded = false;

  /** 
   * pull news items of the channel
   * @fires ready when data is ready
   * @fires load when thumbnails are preloaded (preloadThumbnails option)
   * @fires error on failure
   */
  this.pullNews = function() {
    this.news = [];
    this.isReady = false;
    this.isLoaded = false;
    this.pull({channelID: this.id, lang: this.newsFeed.lang});
    return this;
  };

  this.processData = function(data) {
    this.news = [];
    
    var waiting = 0;

    data.forEach(function(e, i, a) {
      this.news.push(new NewsItem(e.picture, e.thumbnail, e.title, e.lead,
                                  e.body, this));

      if (config.preloadThumbnails) {
        waiting++;
        $(this.news[this.news.length - 1].thumbnail).on('load', function() {
          waiting--;
          if (!waiting) {
            $(this).trigger('load', {channel: this});
            $.trace.info('NewsChannel ' + this.name + ': load');
          }
          this.isLoaded = true;
        }.bind(this));
      } 
    }.bind(this));
    this.isReady = true; 
    $.trace.info('NewsChannel ' + this.name + ': ready');
    if (!waiting) {
      $(this).trigger('load', {channel: this});
      $.trace.info('NewsChannel ' + this.name + ': load');
    }
    return this.news;
  }.bind(this);
};

var NewsItem = function(picture, thumbnail, title, lead, body, channel) {
  this.picture = new Thumbnail(picture, false);
  this.thumbnail = new Thumbnail(picture, config.preloadThumbnails);
  this.title = title;
  this.lead = lead;
  this.body = body;
  this.channel = channel;
};

var Thumbnail = function(url, preload) {
  this.url = url;

  if (preload) {
    this.img = $('<img />').
      attr('src', this.url).
      attr('width', this.width).
      attr('height', this.height)[0];
    $(this.img).on('load', function() {
      $(this).trigger('load', { image: this.img });
      this.isReady = true;
    }.bind(this));
  } else {
    this.isReady = true;
  }
};

TS.Modules.NewsFeed.moduleName = 'NewsFeed';
TS.Modules.NewsFeed.requires = ['Feed'];
})();
