/**
 * latestNews
 * @version 0.1
 * @author Akos Lenkei
 * @version 1.0
 * @author Matyas Buczko
 */

'use strict';

TS.Modules.latestNews = {
	moduleName: 'latestNews',
	autoInit: true,
	config: {},
	focusGraph : null,
	init: function(){

		// delay init until feeds are ready
		if (!TS.Modules.goalFeeds.isReady) {
			$(TS.Modules.goalFeeds).on('ready', this.init.bind(this));
			return;
		}
		
		// loadCss(TS.Config.Path.Style + 'latestNews.css');
		var view = new View(loadTemplate(TS.Config.Path.Modules + this.moduleName + '/' + this.moduleName + TS.Config.templateSuffix), '#latestNews'),
		    viewRef=0,
		    //newsFeed = new TS.Modules.NewsFeed,
		    newsFeed = TS.Modules.goalFeeds.newsFeed,
		    defaultChannel = 0,
		    thumbPreLoader = 0,
		    focusMap = {
		        footerButton: {
		          element: null,
		          jQ: '#latestNews-footerButton',
		          up: 'thumb2',
		        },
						thumb0: {
							element: null,
							jQ : '#latestNews .thumbHolder .thumbElem:eq(0)',
							down:'thumb1',up:'arrow'
						},
						thumb1: {
							element: null,
							jQ : '#latestNews .thumbHolder .thumbElem:eq(1)',
							down:'thumb2',up:'thumb0'
						},
						thumb2: {
							element: null,
							jQ : '#latestNews .thumbHolder .thumbElem:eq(2)',
							up:'thumb1',
							down: 'footerButton'
						},
						arrow: {
							element: null,
							jQ : '#latestNews .arrow.sprite-home-morearrowtop',
							down:'thumb0',
						},
						initial: 'thumb0'
				};
	
		view.set('title', $.translate('latestNews_title'));
		view.set('footerText', $.translate('latestNews_footerText'));

		this.focusGraph = new TS.Modules.focusController.FocusGraph(true);

		this.hide = function(){
		  $(newsFeed).override('ready','latestNewsFeedReady');
		  $(newsFeed).override('error','latestNewsError');
			thumbPreLoader && thumbPreLoader.clear();
		};

		//------------------------------------------------------------------------------------------------------------------ feed error handling
		this.errorState = 0;
		$(newsFeed).override('error','latestNewsError',function(){
			alert('NEWS VIDEOS READY WITH ERROR');
			TS.Modules.latestNews.errorState = 1;
			$('#latestNews').html('');
			viewRef = view.display();
			$("#latestNews").show().children().show();
			$(".thumbHolder",viewRef).html($.translate('latestNews_feeErrorMessage'));
			//$(".footer,.sprite-home-morearrowtop",viewRef).hide();
			$(TS.Modules.latestNews).trigger('ready');
		});

		//------------------------------------------------------------------------------------------------------------------ show
		var show = function(){
			$(newsFeed).override('error','latestNewsError',function(){
				alert('NEWS VIDEOS READY WITH ERROR');
				TS.Modules.latestNews.errorState = 1;
				$('#latestNews').html('');
				viewRef = view.display();
				$("#latestNews").show().children().show();
				$(".thumbHolder",viewRef).html($.translate('latestNews_feeErrorMessage'));
				//$(".footer,.sprite-home-morearrowtop",viewRef).hide();
				$(TS.Modules.latestNews).trigger('ready');
			});

			if(this.subReady)
				return false;

      $('#latestNews').html('');
			if(!newsFeed.channels.length)
			{
				$(newsFeed).off('ready',show).on('ready',show);
				newsFeed.pullChannels();
			}
			else if(!newsFeed.channels[defaultChannel].news.length)
			{
				$(newsFeed.channels[defaultChannel]).off('ready',show).on('ready',show);
				newsFeed.channels[defaultChannel].pullNews();
			}
			else
			{
				renderPage();
			}

		}.bind(this);
		this.show = show;

		//------------------------------------------------------------------------------------------------------------------ renderPage
		var renderPage = function(){
			viewRef = view.display();
			
			var footerButton = $('#latestNews-footerButton');
			
			$("#latestNews").hide();

			thumbPreLoader && thumbPreLoader.clear();
			thumbPreLoader = new TS.Modules.multiPreloader({showOnAllReady : true});
			$(thumbPreLoader).on('imgError',function(e,error){
				error.target.hide();
			});
			
			// populate newsItems
			var newsData = [],
			    news = newsFeed.channels[defaultChannel].news;
			
			for(var i=0;i<Math.min(3,news.length);i++)
			{
				newsData.push([
					{key:'imageTag',value:'<img src="'+thumbPreLoader.add(news[i].thumbnail.url)+'" class="preImage" />'},
					{key:'description',value:news[i].lead},
					{key:'title',value:news[i].title},
					{key:'navigate',value:defaultChannel+'/'+i}
				]);
			}
			thumbPreLoader.preLoad();

			$(thumbPreLoader).on('ready', function() {
				// append correct aspect ratio to all thumbnails
				$('.imgMask img', viewRef).each(function(i, e) {
					var image = $(e);
					var container = $(e).parent();

					var imageRatio = image.prop('naturalWidth') / image.prop('naturalHeight');
			    var containerRatio = container.width() / container.height();

			    if (imageRatio <= containerRatio) {
			    	image.addClass('portrait');
			    } else {
			    	image.addClass('landscape');
			    }
				})
			})
      
			view.foreach(newsData, '#latestNews-Elem');
			setTimeout(function(){
			  $("#latestNews .textHolder").dotdotdot()
			},1000);
			
			// setup focusgraph
			for(var i in focusMap)
			{
				if(focusMap[i].jQ)
				{
					focusMap[i].element = $(focusMap[i].jQ);
				}
			}

			//$('.thumbHolder')TS.Modules.latestNews

			this.focusGraph.setElement($("#latestNews")).setMap(focusMap);
			$("#latestNews").show().children().show();
			alert('LATEST NEWS READY WITHOUT ERROR');
			$(this).trigger('ready');
		}.bind(this);
  }
}

